#!/bin/sh

# UpStare is a system that can apply immediate dynamic software updates
# in multi-threaded userspace applications using stack
# reconstruction. It is Copyright (C) 2006 by Kristis Makris
# <kristis.makris@asu.edu,mkgnu@mkgnu.net>.

# In executable binary form produced and digitally signed specifically
# by Kristis Makris, it is freely distributed for personal and
# academic/research use. It is NOT distributed for commercial use. No
# other binary forms can be distributed.

# The source code and documentation, either in electronic or hardcopy
# format, may not be used for any purpose whatsoever without the written
# permission of Kristis Makris.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


RUNTIME_HEADERS="/usr/include/upstare/runtime"
HCU_BUILD_PATCH="/usr/bin/hcu_cilly.sh --save-temps --stats -I $RUNTIME_HEADERS"
HCU_SMALL_FILE="/usr/lib/upstare/lib/UpStare/share/tools/hcu_small_file.c"

usage()
{
    echo "usage: $0 <old_version_filename> <old_version_number> <new_version_filename> <new_version_number> <mappings_filename> <base_version_number> <support_dynamic_stack_tracing-yes_or_no> <support_blocking_system_calls-yes_or_no>"
    exit -1
}

if [ $# -lt 7 ]; then
    usage
fi

HCU_OLD_VERSION_FILENAME=$1
HCU_OLD_VERSION_NUMBER=$2
HCU_NEW_VERSION_FILENAME=$3
HCU_NEW_VERSION_NUMBER=$4
HCU_MAPPINGS_FILENAME=$5
HCU_BASE_VERSION_NUMBER=$6
HCU_SUPPORT_DYNAMIC_STACK_TRACING=$7
HCU_SUPPORT_BLOCKING_SYSTEM_CALLS=$8

HCU_OUTPUT_FILENAME=${HCU_OLD_VERSION_FILENAME}_v${HCU_OLD_VERSION_NUMBER}_to_v${HCU_NEW_VERSION_NUMBER}.hcupatch.c



# First create preprocessed versions of the files that will be parsed
HCU_PREPROCESSED_OLD_VERSION_FILENAME=$HCU_OLD_VERSION_FILENAME.hcudiff.c
HCU_PREPROCESSED_NEW_VERSION_FILENAME=$HCU_NEW_VERSION_FILENAME.hcudiff.c
HCU_PREPROCESSED_MAPPINGS_FILENAME=$HCU_MAPPINGS_FILENAME.hcudiff.c
$HCU_BUILD_PATCH -E $HCU_OLD_VERSION_FILENAME -o $HCU_PREPROCESSED_OLD_VERSION_FILENAME 
$HCU_BUILD_PATCH -E $HCU_NEW_VERSION_FILENAME -o $HCU_PREPROCESSED_NEW_VERSION_FILENAME 
$HCU_BUILD_PATCH -E $HCU_MAPPINGS_FILENAME -o $HCU_PREPROCESSED_MAPPINGS_FILENAME 

# Now diff the preprocessed files
$HCU_BUILD_PATCH -S --doPatchGeneration --patch-filename=$HCU_OUTPUT_FILENAME --old-version-filename=$HCU_PREPROCESSED_OLD_VERSION_FILENAME --new-version-filename=$HCU_PREPROCESSED_NEW_VERSION_FILENAME --old-version-number=$HCU_OLD_VERSION_NUMBER --new-version-number=$HCU_NEW_VERSION_NUMBER --mappings-filename=$HCU_PREPROCESSED_MAPPINGS_FILENAME --base-version-number=$HCU_BASE_VERSION_NUMBER --support-dynamic-safety=$HCU_SUPPORT_DYNAMIC_STACK_TRACING --support-blocking-system-calls=$HCU_SUPPORT_BLOCKING_SYSTEM_CALLS $HCU_SMALL_FILE



