#!/usr/bin/perl -w

# UpStare is a system that can apply immediate dynamic software updates
# in multi-threaded userspace applications using stack
# reconstruction. It is Copyright (C) 2006 by Kristis Makris
# <kristis.makris@asu.edu,mkgnu@mkgnu.net>.

# In executable binary form produced and digitally signed specifically
# by Kristis Makris, it is freely distributed for personal and
# academic/research use. It is NOT distributed for commercial use. No
# other binary forms can be distributed.

# The source code and documentation, either in electronic or hardcopy
# format, may not be used for any purpose whatsoever without the written
# permission of Kristis Makris.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.


use strict;
use Data::Dumper;

my $cilly = '/usr/bin/hcu_cilly.sh';
my $hcu_transform = "$cilly --save-temps --stats --noPrintLn";

main ( @ARGV );



sub main {
    my ( @arguments ) = ( @_ );

    my $hcu_args = join ( " ", @arguments );

    my $cmd = "$hcu_transform $hcu_args";

    # Try to escape double quotes (") that are not already escaped
    # with a '\'. Otherwise the arguments supplied to the command-line
    # will not be passed as originally intented: the single quotes (')
    # will be dropped. For example, an argument like:
    #
    #      -DSYSCONFDIR="/usr/local/pgsql/etc"
    #
    # will be passed for execution as
    #
    #      -DSYSCONFDIR=/usr/local/pgsql/etc
    #
    # which will cause the variable to no longer hold a string. But
    # Don't escape the double quote if it is already escaped, like:
    #
    # -DPKGLIBDIR=\"/tmp/postgresbuild/lib\"
    #
    # since this will be passed for execution as
    #
    # -DPKGLIBDIR="/tmp/postgresbuild/lib"
    #
    # which is what we want
    $cmd =~ s/([^\\])\"/$1\\\"/g;

    `$cmd`;
}
