/*
UpStare is a system that can apply immediate dynamic software updates
in multi-threaded userspace applications using stack
reconstruction. It is Copyright (C) 2006 by Kristis Makris
<kristis.makris@asu.edu,mkgnu@mkgnu.net>.

In executable binary form produced and digitally signed specifically
by Kristis Makris, it is freely distributed for personal and
academic/research use. It is NOT distributed for commercial use. No
other binary forms can be distributed.

The source code and documentation, either in electronic or hardcopy
format, may not be used for any purpose whatsoever without the written
permission of Kristis Makris.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HCU_HEADERS_H
#define HCU_HEADERS_H



#include "hcu_header_macros.h"
extern int hcu_may_need_to_reconstruct_callee_stack HCU_VERSION_HAS_BEEN_SET;
extern int hcu_must_update HCU_VERSION_HAS_BEEN_SET;



#include "hcu_stack_frame_fields.h"


void hcu_bug(char *error, ...);
void hcu_save_this_frame(char *function_name, void *stack_value, long stack_value_size, hcu_stack_frame_fields_t *field_values, int switch_value);
void hcu_restore_this_frame(char *function_name, void *stack_value, void *params_value, int *switch_value);
int hcu_try_to_update_top_singlethreaded(char *from_function, void *local_stack_variable, long local_stack_variable_size, hcu_stack_frame_fields_t *local_stack_frame_fields, long continuation_value, long update_point_number);
void hcu_try_to_update_bottom_singlethreaded();
int hcu_try_to_update_top_multithreaded(char *from_function, void *local_stack_variable, long local_stack_variable_size, hcu_stack_frame_fields_t *local_stack_frame_fields, long continuation_value, long update_point_number);
void hcu_try_to_update_bottom_multithreaded();



#define _GNU_SOURCE 1
#include "pthread.h"
#include "hcu_stack_trace.h"
#include "hcu_datatypes.h"
#include "hcu_safety_constraints.h"



/* Enumeration of the possible update models */
typedef enum {
  HCU_UPDATE_MODEL_IMMEDIATE = 1,
  HCU_UPDATE_MODEL_LAZY = 2,
} hcu_update_model_t ;



extern int hcu_start_update_thread HCU_VERSION_HAS_BEEN_SET;
extern volatile int hcu_executing_in_signal_handler HCU_VERSION_HAS_BEEN_SET;
extern volatile int hcu_executing_in_function_pointer_parameter_to_library HCU_VERSION_HAS_BEEN_SET;
extern int hcu_stack_trace_enabled HCU_VERSION_HAS_BEEN_SET;



void hcu_init(long version, long number_of_update_points, hcu_function_datatype_description_t (*datatype_descriptions)[], long *datatypes_size, long enable_dynamic_safety);
void hcu_exit();
int hcu_must_reconstruct_callee_stack();
int hcu_must_unwind_up(char *current_function);
void hcu_try_to_update_stub(long line_number);
void hcu_execution_trace(int block_number, char *from_function);
void hcu_set_update_mode_off();
void hcu_set_update_mode_on();
void hcu_static_request_update(hcu_update_model_t update_model);



#include "hcu_thread_list.h"
#include "hcu_mappings.h"
#include <errno.h>



extern hcu_table_t *hcu_blocked_list;
extern int hcu_update_coordinator;
extern int hcu_stack_unwind_up_coordinator;
extern int hcu_stack_reconstruction_coordinator;
extern hcu_lock_t hcu_update_coordinator_lock;
extern hcu_lock_t hcu_stack_unwind_up_coordinator_lock;
extern hcu_lock_t hcu_stack_reconstruction_coordinator_lock;
extern hcu_lock_t hcu_thread_blocker_update_lock;
extern hcu_lock_t hcu_thread_blocker_stack_unwind_up_lock;
extern hcu_lock_t hcu_thread_blocker_stack_reconstruction_lock;
extern struct timeval hcu_time_update_requested;
extern struct timeval hcu_time_must_update_flag_raised;
extern hcu_lock_t hcu_apply_update_lock;


#define HCU_STATIC_UPDATE_INFORMATION_NOT_USED 1
#define HCU_STATIC_UPDATE_INFORMATION_USED 2



#include <sys/types.h>
#include <sys/socket.h>
pid_t hcu_fork();
pid_t hcu_wait(int *status);
pid_t hcu_waitpid(pid_t pid, int *status, int options);
int hcu_would_block();
void hcu_issue_read(int fd, void *buf, size_t count);
void hcu_issue_recv(int fd, void *buf, size_t len, int flags);
void hcu_issue_accept(int fd, struct sockaddr *addr, socklen_t *addrlen);
int hcu_issue_select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
ssize_t hcu_issue_bottom_read(int fd, void *buf, size_t count);
ssize_t hcu_issue_bottom_recv(int fd, void *buf, size_t len, int flags);
int hcu_issue_bottom_accept(int fd, struct sockaddr *addr, socklen_t *addrlen);
#include <string.h>



#endif /*  HCU_HEADERS_H */
