/*
UpStare is a system that can apply immediate dynamic software updates
in multi-threaded userspace applications using stack
reconstruction. It is Copyright (C) 2006 by Kristis Makris
<kristis.makris@asu.edu,mkgnu@mkgnu.net>.

In executable binary form produced and digitally signed specifically
by Kristis Makris, it is freely distributed for personal and
academic/research use. It is NOT distributed for commercial use. No
other binary forms can be distributed.

The source code and documentation, either in electronic or hardcopy
format, may not be used for any purpose whatsoever without the written
permission of Kristis Makris.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HCU_MAPPINGS_H
#define HCU_MAPPINGS_H



typedef long hcu_mapping_execution_continuation_array_size_t;



typedef struct {
  /** Execution point in the original function */
  long original;
  /** Execution point in the equivalent function that the original
      point maps to. */
  long equivalent;
} hcu_mapping_execution_continuation_t;



typedef struct {
  /** Name of thread which will be updated. */
  char thread_name[128];
  /** Name of top-most function in the thread from which stack will be
      reconstructed and the update will be effective. */
  char function_name[128];
} hcu_mapping_update_description_t;



typedef struct {
  /** Name of function whose indirection pointer will be reset. */
  char indirection_pointer_name[128];
  /** Name of function the indirection pointer will be set to. */
  char new_function_name[128];
  /** Flags whether this is an indirection pointer for a thread
      entrypoint, like the main function and functions supplied as
      arguments to pthread_create */
  int is_thread_entrypoint;
} hcu_mapping_function_update_description_t;



typedef struct {
  /** Name of old function name which will be updated. */
  char old[128];
  /** Name of new function name from which execution will continue. */
  char new[128];
  /* Number of mappings from update points to continuation points. */
  hcu_mapping_execution_continuation_array_size_t mappings_size;
  hcu_mapping_execution_continuation_t mappings_array[64];
} hcu_mapping_algorithmic_equivalence_t;



#endif /* HCU_MAPPINGS_H */
