/*
UpStare is a system that can apply immediate dynamic software updates
in multi-threaded userspace applications using stack
reconstruction. It is Copyright (C) 2006 by Kristis Makris
<kristis.makris@asu.edu,mkgnu@mkgnu.net>.

In executable binary form produced and digitally signed specifically
by Kristis Makris, it is freely distributed for personal and
academic/research use. It is NOT distributed for commercial use. No
other binary forms can be distributed.

The source code and documentation, either in electronic or hardcopy
format, may not be used for any purpose whatsoever without the written
permission of Kristis Makris.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HCU_SAFETY_CONSTRAINTS_H
#define HCU_SAFETY_CONSTRAINTS_H


typedef struct {
  /** Minimum execution point where execution might be. Execution must
      have passed this point to match the constraint.

      Note that a minimum of -1 is special and means 'anywhere' (from
      the beginning of the function).
 */
  long minimum;
  /** Maximum execution point where execution might be. Execution must
      have not yet encountered this point to match the constraint.

      Note that a minimum of -1 is special and means 'anywhere' (until
      the end of the function).
  */
  long maximum;
  /** Whether this constraint is a negation (NOT) */
  short is_negation;
} hcu_safety_constraint_item_t;



typedef struct {
  /** Name of thread for which this constraint is applied.

      Note that the name of '*' is special and means all threads. */
  char thread_name[128];
  /** Name of function for which the constraint applies. */
  char function_name[128];
  /** A table that describes the constraints that apply to this
      function of this thread. */
  long constraints_size;
  /* An array of safety constraints in disjunctive normal form. 
  e.g. the constraint:

  (a|b)&c

  should be expressed as:

  c&a|c&b

  This means one should declare an hcu_safety_constraint_t table with
  two elements. In the constraints_array of element 0 one should add
  the two items: c a. In the constraints_array of element 1 one should
  add the two items: c b.
  */
  hcu_safety_constraint_item_t constraints_array[64];
} hcu_safety_constraint_t;



#endif /* HCU_SAFETY_CONSTRAINTS_H */
