/*
UpStare is a system that can apply immediate dynamic software updates
in multi-threaded userspace applications using stack
reconstruction. It is Copyright (C) 2006 by Kristis Makris
<kristis.makris@asu.edu,mkgnu@mkgnu.net>.

In executable binary form produced and digitally signed specifically
by Kristis Makris, it is freely distributed for personal and
academic/research use. It is NOT distributed for commercial use. No
other binary forms can be distributed.

The source code and documentation, either in electronic or hardcopy
format, may not be used for any purpose whatsoever without the written
permission of Kristis Makris.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HCU_STACK_TRACE_H
#define HCU_STACK_TRACE_H



#include <pthread.h>
#include <limits.h>




/* The maximum stack depth we have seen so far is 35 for PostgreSQL
   7.4.16 */
#define HCU_STACK_TRACE_MAXIMUM_DEPTH 256



typedef struct {
  long execution_points[HCU_STACK_TRACE_MAXIMUM_DEPTH];
  char *function_names[HCU_STACK_TRACE_MAXIMUM_DEPTH];
  /* Note that a counter value of 0 means one element. This is so we
     can efficiently save the function_names and execution_points
     arrays without needing to subtract one. */
  long counter;
  char *thread_name;
} hcu_stack_trace_t;



extern pthread_key_t hcu_stack_trace_key;



void hcu_stack_trace_init();
void hcu_stack_trace_thread_init(char *thread_name);
void hcu_stack_trace_thread_exit();
void hcu_stack_trace_clean_otherthreads();
void hcu_print_stack_traces();



#endif /* HCU_STACK_TRACE_H */
