/*
UpStare is a system that can apply immediate dynamic software updates
in multi-threaded userspace applications using stack
reconstruction. It is Copyright (C) 2006 by Kristis Makris
<kristis.makris@asu.edu,mkgnu@mkgnu.net>.

In executable binary form produced and digitally signed specifically
by Kristis Makris, it is freely distributed for personal and
academic/research use. It is NOT distributed for commercial use. No
other binary forms can be distributed.

The source code and documentation, either in electronic or hardcopy
format, may not be used for any purpose whatsoever without the written
permission of Kristis Makris.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef HCU_UPDATE_POINTS_H
#define HCU_UPDATE_POINTS_H



#include <sys/time.h>



typedef struct {
  int enabled;
  /* Flags if the update point has been visited at least once */
  int visited;
  /* Name of the function that contains the update point */
  char *in_function;
  /* Tracks the time the update point was last visited (only when
     hcu_must_update is enabled) */
  struct timeval time_of_last_visit;
  /* Tracks the longest duration of time before the update point was
     revisited (only when hcu_must_update is enabled) */
  struct timeval duration_of_longest_revisit;
} hcu_update_point_t;



extern long hcu_number_of_update_points;
extern int hcu_track_update_points;



void hcu_update_points_init(hcu_update_point_t **hcu_update_points);
void hcu_update_points_visit(long update_point_number, char *from_function, hcu_update_point_t *hcu_update_points);
void hcu_update_points_destroy(hcu_update_point_t *hcu_update_points);



#endif /* HCU_UPDATE_POINTS_H */
